/**
 * Returns the inverse tangent of x in degrees
 * @param {*} x The number to take the inverse tangent of 
 * @returns Inverse tangent of x in degrees
 */
function datan(x){
    return Math.atan(x)*180/Math.PI
}
exports.datan = datan

/**
 * Returns the tangent of x in degrees
 * @param {*} x The number to take the tangent of
 * @returns Tangent of x in degrees
 */
exports.dtan = function(x){
    return Math.tan(x*Math.PI/180)
}

/**
 * Returns ther inverse sine of x in degrees
 * @param {*} x The number to take inverse sine of
 * @returns Inverse sine of x in degrees
 */
 exports.dasin = function(x){
    return Math.asin(x)*180/Math.PI 
}

/**
 * Returns the sine of x in degrees
 * @param {*} x The number to take the sine of
 * @returns The sine of x in degrees
 */
 exports.dsin = function(x){
    return Math.sin(x*Math.PI/180)
}

/**
 * Returns the inverse cosine of x in degrees
 * @param {*} x The number to take the inverse cosine of
 * @returns The inverse cosine of x in degrees
 */
 exports.dacos = function(x){
    return Math.acos(x)*180/Math.PI
}

/**
 * Returns the cosine of x in degrees
 * @param {*} x The number to take the cosine of
 * @returns The cosine of x in degrees
 */
 exports.dcos = function(x){
    return Math.cos(x*Math.PI/180)
}


/**
 * *Vector object
 * @param {*} x the x component of the vector
 * @param {*} y the y component of the vector
 */
exports.vector = class{
    constructor(x, y){
        var me = this
        me.xcom = x
        me.ycom = y
        me.angle = datan(me.ycom/me.xcom)
    }

    /**
     * Calculates the magnitude of the vector
     * @returns The magnitude of the vector
     */
    getMagnitude(){
        return Math.sqrt((me.xcom * me.xcom) + (me.ycom * me.ycom))
    }

    /**
     * Adds components to the vector
     * @param {*} xc x component to add
     * @param {*} yc y component to add
     */
    addComponents(xc, yc){
        me.xcom += xc
        me.ycom += yc
        me.angle = datan(me.ycom/me.xcom)
    }

    /**
     * Add another vector
     * @param {*} othervec Other vector to add
     */
    addVector(othervec){
        me.xcom += othervec.xcom
        me.ycom += othervec.ycom
        me.angle = datan(me.ycom/me.xcom)
    }

    /**
     * Subtract another vector
     * @param {*} othervec Other vector to subtract
     */
    subVector(othervec){
        me.xcom -= othervec.xcom
        me.ycom -= othervec.ycom
        me.angle = datan(me.ycom/me.xcom)
    }

    /**
     * Print the vector to the console
     */
    print(){
        console.log("xcom:"+me.xcom+" ycom:"+me.ycom+" angle:"+me.angle+" magnitude:"+me.getMagnitude())
    }

    /**
     * Reflects the vector horizontally
     */
    xReflect(){
        me.ycom *= -1
        me.angle = datan(me.ycom/me.xcom)
    }

    /**
     * Reflects the vector vertically
     */
    yReflect(){
        me.xcom *= -1
        me.angle = datan(me.ycom/me.xcom)
    }
}
/**
 * Creates a new vector of specified angle and magnitude
 * @param {*} angle Angle of new vector
 * @param {*} magnitude Magitude of new vector
 * @returns A new vector of given angle and magnitude
 */
function createVectorOfAngleAndMagnitude(angle, magnitude){
    xx = magnitude*dcos(angle)
    yy = magnitude*dsin(angle)
    nm = new vector(xx, yy)
    return nm
}

/**
 * *Orb Object
 * @param {*} mass Mass of the orb
 * @param {*} radius Radius of the orb
 */
class orb{
    constructor(mass, radius, max_trace){
        this.mass = mass //orb mass
        this.velocity = new vector(0, 0) //orb velocity
        this.xpos = 0 //orb x position
        this.ypos = 0 //orb y position
        this.radius = radius //orb radius
        this.exterminated = false //debug variable
        this.trace = false //FIXME whether the orbs path will be traces

        this.trace_mat_y = [] //array of previous y positions
        this.trace_mat_x = [] //array of previous x positions
        this.trace_velocity = [] //array of previous velocities

    }

    /**
     * Changes the position of orb
     * @param {*} x new x position of orb
     * @param {*} y new y position of orb
     */
    position(x, y){
        this.xpos = x
        this.ypos = y
    }

    /**
     * moves the orb in the direction of velocity
     * adds velocity and position to trace arrays
     */
    move(){
        this.xpos += this.velocity.xcom
        this.ypos += this.velocity.ycom

        this.trace_mat_x.push(this.xpos)
        this.trace_mat_y.push(this.ypos)
        this.trace_velocity.push(this.velocity.getMagnitude())
        if(this.trace_mat_x.length>max_trace){
            this.trace_mat_x.splice(0,1)
            this.trace_mat_y.splice(0,1)
            this.trace_velocity.splice(0,1)
            }
        }

    /**
     * Plots the orb to the canvas
     * @param {*} color Color of the orb
     */
    plot(color){
        fill(color)
        ellipse(this.xpos,this.ypos,this.radius)
    }

    /**
     * Displays the orb's velocity
     * @param {*} multiplier Extra length to add to final velocity arrow
     */
    showVelocity(multiplier){
        arrow(this.xpos, this.ypos, this.velocity.xcom, this.velocity.ycom, multiplier)
    }
}

/**
 * Traces orb
 * @param {*} orb orb to trace
 */
//! DOES NOT WORK FOR SOME REASON IDFK WHY
//TODO: Figure out why this doesn't work
function plotTraceorb(orb){
        for(i = 0; i < orb.trace_mat_x.length; i ++){
            stroke(50*orb.trace_velocity[i],-100+50*orb.trace_velocity[i],255)
            fill(50*orb.trace_velocity[i],-100+50*orb.trace_velocity[i],255)
            ellipse(orb.trace_mat_x[i], orb.trace_mat_y[i],5)
        }
}
//!--------------------------------------

/**
 * Calculates the distance between two orbs
 * @param {*} r_orb The orb with current respect
 * @param {*} o_orb The other orb
 * @returns The distance between two orbs
 */
function distanceBetweenTwoorbs(r_orb, o_orb){
    xdis = r_orb.xpos - o_orb.xpos
    ydis = r_orb.ypos - o_orb.ypos
    return Math.sqrt((xdis*xdis)+(ydis*ydis)) 
}

/**
 * Calculates the angle between two orbs
 * @param {*} r_orb The orb with current respect
 * @param {*} o_orb The other orb
 * @returns The angle between two orbs
 */
function angleBetweenorbs(r_orb, o_orb){
    xdis = r_orb.xpos - o_orb.xpos
    ydis = r_orb.ypos - o_orb.ypos
    if (xdis > 0)
        return datan(ydis/xdis)+180
    else{
        return datan(ydis/xdis)
    }
}

/**
 * Calculates the scalar gravitational attraction between two orbs
 * @param {*} r_orb orb with current respect
 * @param {*} o_orb Other orb
 * @returns The scalar gravitational attraction between two orbs
 */
function gravitationalAttractionBetweenTwoOrbs(r_orb, o_orb, grav_constant){
    if(gravConstant=="G"){
        G = 0.00000000006673
    }else{
        G = grav_constant
    }

    r = distanceBetweenTwoorbs(r_orb, o_orb)
    return G*r_orb.mass*o_orb.mass/(r*r)
}

/**
 * 
 * @param {*} x The starting x position of the arrow
 * @param {*} y The starting y position of the arrow
 * @param {*} dx The dx distance from the initial x to the head
 * @param {*} dy The dy distance from the initial y to the head
 * @param {*} length_multiplier Value to make the arrow appear larger or smaller
 */
//TODO Add triangle at the end of the arrow
function arrow(x,y,dx,dy,length_multiplier){
    finalx = x+dx*length_multiplier
    finaly = y+dy*length_multiplier
    stroke(255)
    line(x,y,finalx,finaly)
}